@@ -11,6 +11,7 @@ from datetime import timedelta
 from functools import cached_property
 from os.path import normpath
 from os.path import split
+from typing import Any
 from typing import Iterable
 from uuid import uuid4
 
@@ -51,6 +52,7 @@ class Todo:
     related: list[Todo]
     rrule: str | None
     start: date | None
+    id: int | None
 
     def __init__(
         self,
@@ -58,7 +60,7 @@ class Todo:
         mtime: int | None = None,
         new: bool = False,
         list: TodoList | None = None,
-    ):
+    ) -> None:
         """
         Creates a new todo using `todo` as a source.
 
@@ -112,12 +114,12 @@ class Todo:
         """
         todo = Todo(new=True, list=self.list)
 
-        fields = (
-            Todo.STRING_FIELDS
-            + Todo.INT_FIELDS
-            + Todo.LIST_FIELDS
-            + Todo.DATETIME_FIELDS
-        )
+        fields = [
+            *Todo.STRING_FIELDS,
+            *Todo.INT_FIELDS,
+            *Todo.LIST_FIELDS,
+            *Todo.DATETIME_FIELDS,
+        ]
         fields.remove("uid")
 
         for field in fields:
@@ -125,33 +127,29 @@ class Todo:
 
         return todo
 
-    STRING_FIELDS = [
+    STRING_FIELDS = (
         "description",
         "location",
         "status",
         "summary",
         "uid",
         "rrule",
-    ]
-    INT_FIELDS = [
+    )
+    INT_FIELDS = (
         "percent_complete",
         "priority",
         "sequence",
-    ]
-    LIST_FIELDS = [
-        "categories",
-    ]
-    DATETIME_FIELDS = [
+    )
+    LIST_FIELDS = ("categories",)
+    DATETIME_FIELDS = (
         "completed_at",
         "created_at",
         "dtstamp",
         "start",
         "due",
         "last_modified",
-    ]
-    RRULE_FIELDS = [
-        "rrule",
-    ]
+    )
+    RRULE_FIELDS = ("rrule",)
     ALL_SUPPORTED_FIELDS = (
         DATETIME_FIELDS + INT_FIELDS + LIST_FIELDS + RRULE_FIELDS + STRING_FIELDS
     )
@@ -163,7 +161,7 @@ class Todo:
         "NEEDS-ACTION",
     )
 
-    def __setattr__(self, name: str, value):
+    def __setattr__(self, name: str, value: None | str | int | list) -> None:
         """Check type and avoid setting fields to None"""
         """when that is not a valid attribue."""
 
@@ -211,7 +209,7 @@ class Todo:
     def is_recurring(self) -> bool:
         return bool(self.rrule)
 
-    def _apply_recurrence_to_dt(self, dt) -> datetime | None:
+    def _apply_recurrence_to_dt(self, dt: date | None) -> datetime | None:
         if not dt:
             return None
 
@@ -222,7 +220,7 @@ class Todo:
 
         return recurrence.after(dt)
 
-    def _create_next_instance(self):
+    def _create_next_instance(self) -> Todo:
         copy = self.clone()
         copy.due = self._apply_recurrence_to_dt(self.due)
         copy.start = self._apply_recurrence_to_dt(self.start)
@@ -286,7 +284,7 @@ class VtodoWriter:
         "rrule": "rrule",
     }
 
-    def __init__(self, todo: Todo):
+    def __init__(self, todo: Todo) -> None:
         self.todo = todo
 
     def normalize_datetime(self, dt: date) -> date:
@@ -310,7 +308,7 @@ class VtodoWriter:
 
         return dt.astimezone(pytz.UTC)
 
-    def serialize_field(self, name: str, value):
+    def serialize_field(self, name: str, value: Any) -> int | str | date:
         if name in Todo.RRULE_FIELDS:
             return icalendar.vRecur.from_ical(value)
         if name in Todo.DATETIME_FIELDS:
@@ -324,14 +322,14 @@ class VtodoWriter:
 
         raise Exception(f"Unknown field {name} serialized.")
 
-    def set_field(self, name: str, value):
+    def set_field(self, name: str, value: Any) -> None:
         # If serialized value is None:
         self.vtodo.pop(name)
         if value:
             logger.debug("Setting field %s to %s.", name, value)
             self.vtodo.add(name, value)
 
-    def serialize(self, original=None):
+    def serialize(self, original: icalendar.Todo = None) -> icalendar.Todo:
         """Serialize a Todo into a VTODO."""
         if not original:
             original = icalendar.Todo()
@@ -347,14 +345,15 @@ class VtodoWriter:
 
         return self.vtodo
 
-    def _read(self, path):
+    def _read(self, path: str) -> icalendar.Todo:
         with open(path, "rb") as f:
             cal = f.read()
-            cal = icalendar.Calendar.from_ical(cal)
-            for component in cal.walk("VTODO"):
+            data = icalendar.Calendar.from_ical(cal)
+            for component in data.walk("VTODO"):
                 return component
+            return None
 
-    def write(self):
+    def write(self) -> icalendar.Todo:
         if os.path.exists(self.todo.path):
             self._write_existing(self.todo.path)
         else:
@@ -362,7 +361,7 @@ class VtodoWriter:
 
         return self.vtodo
 
-    def _write_existing(self, path):
+    def _write_existing(self, path: str) -> None:
         original = self._read(path)
         vtodo = self.serialize(original)
 
@@ -375,7 +374,7 @@ class VtodoWriter:
         with AtomicWriter(path, "wb", overwrite=True).open() as f:
             f.write(cal.to_ical())
 
-    def _write_new(self, path):
+    def _write_new(self, path: str) -> icalendar.Todo:
         vtodo = self.serialize()
 
         c = icalendar.Calendar()
@@ -402,9 +401,9 @@ class Cache:
     may be used for filtering/sorting.
     """
 
-    SCHEMA_VERSION = 9
+    SCHEMA_VERSION = 10
 
-    def __init__(self, path: str):
+    def __init__(self, path: str) -> None:
         self.cache_path = str(path)
         os.makedirs(os.path.dirname(self.cache_path), exist_ok=True)
 
@@ -417,7 +416,7 @@ class Cache:
     def save_to_disk(self) -> None:
         self._conn.commit()
 
-    def is_latest_version(self):
+    def is_latest_version(self) -> bool:
         """Checks if the cache DB schema is the latest version."""
         try:
             return self._conn.execute(
@@ -427,7 +426,7 @@ class Cache:
         except sqlite3.OperationalError:
             return False
 
-    def drop_tables(self):
+    def drop_tables(self) -> None:
         self._conn.executescript(
             """
             DROP TABLE IF EXISTS todos;
@@ -437,7 +436,7 @@ class Cache:
         """
         )
 
-    def create_tables(self):
+    def create_tables(self) -> None:
         if self.is_latest_version():
             return
 
@@ -482,6 +481,7 @@ class Cache:
                 "id" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
                 "todos_id" INTEGER NOT NULL,
                 "category" TEXT,
+                "category_upper" TEXT,
 
                 CONSTRAINT category_unique UNIQUE (todos_id,category),
                 FOREIGN KEY(todos_id) REFERENCES todos(id) ON DELETE CASCADE
@@ -518,12 +518,11 @@ class Cache:
         """
         )
 
-    def clear(self):
+    def clear(self) -> None:
         self._conn.close()
         os.remove(self.cache_path)
-        self._conn = None
 
-    def add_list(self, name: str, path: str, colour: str, mtime: int):
+    def add_list(self, name: str, path: str, colour: str | None, mtime: int) -> str:
         """
         Inserts a new list into the cache.
 
@@ -560,7 +559,7 @@ class Cache:
 
         return self.add_list(name, path, colour, mtime)
 
-    def add_file(self, list_name: str, path: str, mtime: int):
+    def add_file(self, list_name: str, path: str, mtime: int) -> None:
         try:
             self._conn.execute(
                 """
@@ -579,16 +578,17 @@ class Cache:
         except sqlite3.IntegrityError as e:
             raise exceptions.AlreadyExistsError("file", list_name) from e
 
-    def add_category(self, todos_id, category):
+    def add_category(self, todos_id: int, category: str) -> None:
         try:
             self._conn.execute(
                 """
                INSERT INTO categories (
                    todos_id,
-                   category
-               ) VALUES (?, ?);
+                   category,
+                   category_upper
+               ) VALUES (?, ?, ?);
                """,
-                (todos_id, category),
+                (todos_id, category, category.upper()),
             )
         except sqlite3.IntegrityError as e:
             raise exceptions.AlreadyExistsError("category", category) from e
@@ -618,18 +618,22 @@ class Cache:
             dt = dt.replace(tzinfo=LOCAL_TIMEZONE)
         return dt.timestamp(), is_date
 
-    def _serialize_rrule(self, todo, field) -> str | None:
+    def _serialize_rrule(self, todo: icalendar.Todo, field: str) -> str | None:
         rrule = todo.get(field)
         if not rrule:
             return None
 
         return rrule.to_ical().decode()
 
-    def add_vtodo(self, todo: icalendar.Todo, file_path: str, id=None) -> int:
-        """
-        Adds a todo into the cache.
+    def add_vtodo(
+        self,
+        todo: icalendar.Todo,
+        file_path: str,
+        id: int | None = None,
+    ) -> int:
+        """Adds a todo into the cache.
 
-        :param icalendar.Todo todo: The icalendar component object on which
+        :param todo: The icalendar component object on which
         """
 
         sql = """
@@ -685,7 +689,7 @@ class Cache:
         ]
 
         if id:
-            params = [id] + params
+            params = [id, *params]
             sql = sql.format("id,\n", "?, ")
         else:
             sql = sql.format("", "")
@@ -706,17 +710,18 @@ class Cache:
 
     def todos(
         self,
-        lists=(),
-        categories=None,
-        priority=None,
-        location="",
-        grep="",
-        sort=(),
-        reverse=True,
-        due=None,
-        start=None,
-        startable=False,
-        status="NEEDS-ACTION,IN-PROCESS",
+        *,
+        lists: Iterable = (),
+        categories: str | None = None,
+        priority: str | None = None,
+        location: str = "",
+        grep: str = "",
+        sort: Iterable = (),
+        reverse: bool = True,
+        due: int | None = None,
+        start: tuple[bool, datetime] | None = None,
+        startable: bool = False,
+        status: str = "NEEDS-ACTION,IN-PROCESS",
     ) -> Iterable[Todo]:
         """
         Returns filtered cached todos, in a specified order.
@@ -728,24 +733,18 @@ class Cache:
             due
             -created_at
 
-        :param list lists: Only return todos for these lists.
-        :param str location: Only return todos with a location containing this
-            string.
-        :param str categories: Only return todos with a category containing this
-            string.
-        :param str grep: Filter common fields with this substring.
-        :param list sort: Order returned todos by these fields. Field names
+        :param lists: Only return todos for these lists.
+        :param location: Only return todos with a location containing this string.
+        :param categories: Only return todos with a category containing this string.
+        :param grep: Filter common fields with this substring.
+        :param sort: Order returned todos by these fields. Field names
             with a ``-`` prepended will be used to sort in reverse order.
-        :param bool reverse: Reverse the order of the todos after sorting.
-        :param int due: Return only todos due within ``due`` hours.
-        :param str priority: Only return todos with priority at least as
-            high as specified.
-        :param tuple(bool, datetime) start: Return only todos before/after
-            ``start`` date
-        :param list(str) status: Return only todos with any of the given
-            statuses.
+        :param reverse: Reverse the order of the todos after sorting.
+        :param due: Return only todos due within ``due`` hours.
+        :param priority: Only return todos with priority at least as high as specified.
+        :param  start: Return only todos before/after ``start`` date
+        :param  status: Return only todos with any of the given statuses.
         :return: A sorted, filtered list of todos.
-        :rtype: generator
         """
         extra_where = []
         params: list = []
@@ -768,7 +767,7 @@ class Cache:
             params.extend(lists)
         if categories:
             category_slots = ", ".join(["?"] * len(categories))
-            extra_where.append(f"AND upper(categories.category) IN ({category_slots})")
+            extra_where.append(f"AND categories.category_upper IN ({category_slots})")
             params = params + [category.upper() for category in categories]
         if priority:
             extra_where.append("AND PRIORITY > 0 AND PRIORITY <= ?")
@@ -788,13 +787,13 @@ class Cache:
             params.append(max_due)
         if start:
             is_before, dt = start
-            dt = dt.timestamp()
+            timestamp = dt.timestamp()
             if is_before:
                 extra_where.append("AND start <= ?")
-                params.append(dt)
+                params.append(timestamp)
             else:
                 extra_where.append("AND start >= ?")
-                params.append(dt)
+                params.append(timestamp)
         if startable:
             extra_where.append("AND (start IS NULL OR start <= ?)")
             params.append(datetime.now().timestamp())
@@ -854,22 +853,21 @@ class Cache:
             seen_paths.add(path)
             yield todo
 
-    def _datetime_from_db(self, dt) -> datetime | None:
+    def _datetime_from_db(self, dt: float) -> datetime | None:
         if dt:
             return datetime.fromtimestamp(dt, LOCAL_TIMEZONE)
         return None
 
-    def _date_from_db(self, dt, is_date=False) -> date | None:
+    def _date_from_db(self, dt: float, is_date: bool = False) -> date | None:
         """Deserialise a date (possible datetime)."""
         if not dt:
-            return dt
+            return None
 
         if is_date:
             return datetime.fromtimestamp(dt, LOCAL_TIMEZONE).date()
-        else:
-            return datetime.fromtimestamp(dt, LOCAL_TIMEZONE)
+        return datetime.fromtimestamp(dt, LOCAL_TIMEZONE)
 
-    def _categories_from_db(self, categories):
+    def _categories_from_db(self, categories: str) -> list[str]:
         if categories:
             return categories.split(",")
 
@@ -926,7 +924,7 @@ class Cache:
     def delete_list(self, name: str) -> None:
         self._conn.execute("DELETE FROM lists WHERE lists.name = ?", (name,))
 
-    def todo(self, id: int, read_only=False) -> Todo:
+    def todo(self, id: int, read_only: bool = False) -> Todo:
         # XXX: DON'T USE READ_ONLY
         result = self._conn.execute(
             """
@@ -973,7 +971,7 @@ class Cache:
 
 
 class TodoList:
-    def __init__(self, name: str, path: str, colour: str | None = None):
+    def __init__(self, name: str, path: str, colour: str | None = None) -> None:
         self.path = path
         self.name = name
         self.colour = colour
@@ -1009,10 +1007,9 @@ class TodoList:
 
         if mtimes:
             return max(mtimes)
-        else:
-            return 0
+        return 0
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: object) -> bool:
         if isinstance(other, TodoList):
             return self.name == other.name
         return object.__eq__(self, other)
@@ -1030,7 +1027,7 @@ class Database:
     classes.
     """
 
-    def __init__(self, paths, cache_path):
+    def __init__(self, paths: Iterable[str], cache_path: str) -> None:
         self.cache = Cache(cache_path)
         self.paths = [str(path) for path in paths]
         self.update_cache()
@@ -1078,10 +1075,10 @@ class Database:
 
         self.cache.save_to_disk()
 
-    def todos(self, **kwargs) -> Iterable[Todo]:
+    def todos(self, **kwargs: Any) -> Iterable[Todo]:
         return self.cache.todos(**kwargs)
 
-    def todo(self, id: int, **kwargs) -> Todo:
+    def todo(self, id: int, **kwargs: Any) -> Todo:
         return self.cache.todo(id, **kwargs)
 
     def lists(self) -> Iterable[TodoList]:
@@ -1107,7 +1104,7 @@ class Database:
                 self.delete(todo)
 
         self.cache.clear()
-        self.cache = None
+        self.cache = Cache(self.cache.cache_path)
 
     def save(self, todo: Todo) -> None:
         if not todo.list:
